@@ -4,12 +4,10 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
-"""Module to handle downloads for different types of Firefox and Thunderbird builds."""
-
-
 from datetime import datetime
 from optparse import OptionParser, OptionGroup
 import os
+import pkg_resources
 import re
 import sys
 import time
@@ -21,6 +19,16 @@ import mozinfo
 from parser import DirectoryParser
 from timezones import PacificTimezone
 
+import progressbar
+
+version = pkg_resources.require("mozdownload")[0].version
+
+__doc__= """
+Module to handle downloads for different types of ftp.mozilla.org hosted 
+applications.
+
+mozdownload version: %(version)s
+""" % {'version' : version}
 
 APPLICATIONS = ['b2g', 'firefox', 'thunderbird']
 
@@ -48,12 +56,20 @@ class NotFoundException(Exception):
         Exception.__init__(self, ': '.join([message, location]))
 
 
+class TimeoutException(Exception):
+    """Exception for a download exceeding the allocated timeout"""
+    def __init__(self):
+        self.message = 'The download exceeded the allocated timeout'
+        Exception.__init__(self, self.message)
+
+
 class Scraper(object):
     """Generic class to download an application from the Mozilla server"""
 
     def __init__(self, directory, version, platform=None,
                  application='firefox', locale='en-US', extension=None,
-                 authentication=None, retry_attempts=3, retry_delay=10):
+                 authentication=None, retry_attempts=0, retry_delay=10,
+                 timeout=180):
 
         # Private properties for caching
         self._target = None
@@ -67,6 +83,7 @@ class Scraper(object):
         self.authentication = authentication
         self.retry_attempts = retry_attempts
         self.retry_delay = retry_delay
+        self.timeout = timeout
 
         # build the base URL
         self.application = application
@@ -76,28 +93,39 @@ class Scraper(object):
     @property
     def binary(self):
         """Return the name of the build"""
+        
+        attempt = 0
 
-        if self._binary is None:
-            # Retrieve all entries from the remote virtual folder
-            parser = DirectoryParser(self.path)
-            if not parser.entries:
-                raise NotFoundException('No entries found', self.path)
-    
-            # Download the first matched directory entry
-            pattern = re.compile(self.binary_regex, re.IGNORECASE)
-            for entry in parser.entries:
-                try:
-                    self._binary = pattern.match(entry).group()
-                    break
-                except:
-                    # No match, continue with next entry
-                    continue
+        while self._binary is None:
+            attempt += 1
+            try:
+                # Retrieve all entries from the remote virtual folder
+                parser = DirectoryParser(self.path)
+                if not parser.entries:
+                    raise NotFoundException('No entries found', self.path)
 
-        if self._binary is None:
-            raise NotFoundException("Binary not found in folder", self.path)
-        else:
-            return self._binary
+                # Download the first matched directory entry
+                pattern = re.compile(self.binary_regex, re.IGNORECASE)
+                for entry in parser.entries:
+                    try:
+                        self._binary = pattern.match(entry).group()
+                        break
+                    except:
+                        # No match, continue with next entry
+                        continue
+                else:
+                    raise NotFoundException("Binary not found in folder",
+                                            self.path)
+            except (NotFoundException, urllib2.HTTPError):
+                if self.retry_attempts > 0:
+                    # Print only if multiple attempts are requested
+                    print "Binary not found! Retrying... (attempt %s)" % attempt
+                if attempt >= self.retry_attempts:
+                    raise NotFoundException("Binary not found in folder",
+                                            self.path)
+                time.sleep(self.retry_delay)
 
+        return self._binary
 
     @property
     def binary_regex(self):
@@ -163,7 +191,7 @@ class Scraper(object):
     def download(self):
         """Download the specified file"""
 
-        attempts = 0
+        attempt = 0
 
         if not os.path.isdir(self.directory):
             os.makedirs(self.directory)
@@ -189,19 +217,36 @@ class Scraper(object):
             urllib2.install_opener(opener)
 
         while True:
-            attempts += 1
+            attempt += 1
             try:
+                start_time = datetime.now()
                 r = urllib2.urlopen(self.final_url)
+                total_size = int(r.info().getheader('Content-length').strip())
                 CHUNK = 16 * 1024
+                # ValueError: Value out of range if only total_size given
+                max_value = ((total_size / CHUNK) + 1) * CHUNK
+                bytes_downloaded = 0
+                widgets = [progressbar.Percentage(), ' ', progressbar.Bar(),
+                           ' ', progressbar.ETA(), ' ', progressbar.FileTransferSpeed()]
+                pbar = progressbar.ProgressBar(widgets=widgets, maxval=max_value).start()
+
                 with open(tmp_file, 'wb') as f:
                     for chunk in iter(lambda: r.read(CHUNK), ''):
                         f.write(chunk)
+                        t1 = (datetime.now() - start_time).total_seconds()
+                        bytes_downloaded += CHUNK
+                        pbar.update(bytes_downloaded)
+                        if t1 >= self.timeout:
+                            raise TimeoutException
+                pbar.finish()
                 break
-            except (urllib2.HTTPError, urllib2.URLError):
+            except (urllib2.HTTPError, urllib2.URLError, TimeoutException):
                 if tmp_file and os.path.isfile(tmp_file):
                     os.remove(tmp_file)
-                print 'Download failed! Retrying... (attempt %s)' % attempts
-                if attempts >= self.retry_attempts:
+                if self.retry_attempts > 0:
+                    # Print only if multiple attempts are requested
+                    print '\nDownload failed! Retrying... (attempt %s)' % attempt
+                if attempt >= self.retry_attempts:
                     raise
                 time.sleep(self.retry_delay)
 
@@ -683,7 +728,6 @@ def cli():
                            'current working directory')
     parser.add_option('--build-number',
                       dest='build_number',
-                      default=None,
                       type="int",
                       metavar='BUILD_NUMBER',
                       help='Number of the build (for candidate, daily, '
@@ -706,9 +750,10 @@ def cli():
                       help='Type of build to download, default: "%default"')
     parser.add_option('--url',
                       dest='url',
-                      default=None,
                       metavar='URL',
-                      help='URL to download.')
+                      help='URL to download. Note: Reserved characters (such as &)\
+                           must be escaped or put in quotes otherwise CLI output\
+                           may be abnormal.')
     parser.add_option('--version', '-v',
                       dest='version',
                       metavar='VERSION',
@@ -716,23 +761,20 @@ def cli():
                             candidate builds, i.e. "3.6"')
     parser.add_option('--extension',
                       dest='extension',
-                      default=None,
                       metavar='EXTENSION',
                       help='File extension of the build (e.g. "zip"), default:\
                             the standard build extension on the platform.')
     parser.add_option('--username',
                       dest='username',
-                      default=None,
                       metavar='USERNAME',
                       help='Username for basic HTTP authentication.')
     parser.add_option('--password',
                       dest='password',
-                      default=None,
                       metavar='PASSWORD',
                       help='Password for basic HTTP authentication.')
     parser.add_option('--retry-attempts',
                       dest='retry_attempts',
-                      default=3,
+                      default=0,
                       type=int,
                       metavar='RETRY_ATTEMPTS',
                       help='Number of times the download will be attempted in '
@@ -744,6 +786,13 @@ def cli():
                       metavar='RETRY_DELAY',
                       help='Amount of time (in seconds) to wait between retry '
                            'attempts, default: %default')
+    parser.add_option('--timeout',
+                      dest='timeout',
+                      default=180,
+                      type=int,
+                      metavar='TIMEOUT',
+                      help='Amount of time (in seconds) until download times '
+                           'out, default: %default')
 
     # Option group for candidate builds
     group = OptionGroup(parser, "Candidate builds",
@@ -765,12 +814,10 @@ def cli():
                      help='Name of the branch, default: "%default"')
     group.add_option('--build-id',
                       dest='build_id',
-                      default=None,
                       metavar='BUILD_ID',
                       help='ID of the build to download')
     group.add_option('--date',
                      dest='date',
-                     default=None,
                      metavar='DATE',
                      help='Date of the build, default: latest build')
     parser.add_option_group(group)
@@ -805,7 +852,8 @@ def cli():
                             'username': options.username,
                             'password': options.password},
                         'retry_attempts': options.retry_attempts,
-                        'retry_delay': options.retry_delay}
+                        'retry_delay': options.retry_delay,
+                        'timeout': options.timeout}
     scraper_options = {'candidate': {
                            'build_number': options.build_number,
                            'no_unsigned': options.no_unsigned},
